﻿using System;
using System.Collections.Generic;
using System.Reflection;
using Owin;
using System.IO;

namespace Jws.OwinAdapter.MsOwin
{
    /// <summary>
    /// 有关OWIN配置信息的操作
    /// </summary>
    static class Configuration
    {


        /// <summary>
        /// 从bin文件夹的dll中查出所有OWIN启动方法
        /// <para>暂不支持在web.config获取启动方法</para>
        /// </summary>
        /// <returns></returns>
        public static List<Action<IAppBuilder>> Get()
        {

            var bin = AppDomain.CurrentDomain.SetupInformation.PrivateBinPath;
            if (!Directory.Exists(bin)) return null;

            var ds = new DirectoryInfo(bin);
            var files = ds.GetFiles("*.dll", SearchOption.AllDirectories);
            if (files.Length < 1) return null;

            var result = new List<Action<IAppBuilder>>();

            foreach (var f in files)
            {
                Assembly assembly = null;

                try
                {
                    assembly = AppDomain.CurrentDomain.Load(AssemblyName.GetAssemblyName(f.FullName));
                }
                catch (BadImageFormatException) { continue; }
                catch (Exception e) { continue; }


                var dconfig = SearchConfigurationFromAssembly(assembly);
                if (dconfig == null) continue;

                foreach (var d in dconfig)
                {

                    var dm = d.Item1.GetMethod(d.Item2, new[] { typeof(IAppBuilder) });
                    if (dm == null || dm.ReturnType != typeof(void)) continue;
                    object dc = null;
                    try
                    {
                        dc = Activator.CreateInstance(d.Item1);
                    }
                    catch { continue; }

                    var dd = Delegate.CreateDelegate(typeof(Action<IAppBuilder>), dc, dm) as Action<IAppBuilder>;
                    if (dd == null) continue;
                    result.Add(dd);
                }

                continue;
            }

            return result.Count < 1 ? null : result;
        }




        /// <summary>
        /// 从Assembly的自定义属性值中获取配置情况
        /// <param name="assembly">打开的程序集</param>
        /// <param name="friendlyName">Web.config中定义的友好名称</param>
        /// </summary>
        /// <returns>返回二元组类型的数组，元素一点是类，结点二表示方法名称</returns>
        static IEnumerable<Tuple<Type, string>> SearchConfigurationFromAssembly(Assembly assembly, string friendlyName = "")
        {

            var listConfig = new List<Tuple<Type, string>>();

            /////////////////////////////////////////////////////////////
            // OwinStartupAttribute包含如下字段名：
            // FriendlyName ：友好名称（在web.config中定义的友好名称）
            // MethodName   ：方法名称
            // StartupType  ：含有方法的类
            /////////////////////////////////////////////////////////////

            foreach (var o in assembly.GetCustomAttributes(false))
            {
                var attrTyp = o.GetType();
                if (!attrTyp.Name.Equals("OwinStartupAttribute", StringComparison.Ordinal)) continue;

                var startupTypeProperty = attrTyp.GetProperty("StartupType", typeof(Type));
                if (startupTypeProperty == null) continue;

                var startupType = startupTypeProperty.GetValue(o, null) as Type;
                if (startupType == null) continue;

                string friendlyNameValue = string.Empty;
                if (!string.IsNullOrWhiteSpace(friendlyName))
                {
                    var friendlyNameProperty = attrTyp.GetProperty("FriendlyName", typeof(string));
                    if (friendlyNameProperty != null)
                    {
                        friendlyNameValue = friendlyNameProperty.GetValue(o, null) as string ?? string.Empty;
                    }
                    if (!string.Equals(friendlyName, friendlyNameValue, StringComparison.OrdinalIgnoreCase)) continue;
                }

                var methodName = string.Empty;
                var methodNameProperty = attrTyp.GetProperty("MethodName", typeof(string));
                if (methodNameProperty != null)
                {
                    methodName = methodNameProperty.GetValue(o, null) as string ?? string.Empty;
                }
                if (methodName == string.Empty) methodName = "Configuration";
                var fullMatch = new Tuple<Type, string>(startupType, methodName);

                listConfig.Add(fullMatch);

            }

            if (listConfig.Count > 0) return listConfig;

            //如果从程序集定义中没有找到，就按类名和方法名找
            var assemblyTypes = assembly.GetTypes();
            if (assemblyTypes.Length < 0) return null;


            /////////////////////////////////////////
            // 启动类的类名是：Startup
            // 启动类的方法是：Configuration
            // Configuration方法应该是公开的
            /////////////////////////////////////////

            foreach (var type in assemblyTypes)
            {
                if (type.Name == "Startup")
                {
                    var configMethod = type.GetMethod("Configuration", new[] { typeof(IAppBuilder) });

                    if (configMethod == null) continue;

                    if (configMethod.ReturnType != typeof(void)) continue;

                    listConfig.Add(new Tuple<Type, string>(type, "Configuration"));
                }
            }

            return (listConfig.Count > 0) ? listConfig : null;

        }

    }
}
